// GUI_Main - WxWidgets GUI for MediaInfo
// Copyright (C) 2002-2007 Jerome Martinez, Zen@MediaArea.net
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//---------------------------------------------------------------------------
#include "GUI/WxWidgets/GUI_Main.h"
#include "Common/Core.h"
#include "GUI/WxWidgets/GUI_Main_FileDrop.h"
#include <wx/html/htmlwin.h>
#include <wx/file.h>
#include <wx/dnd.h>
#include <wx/fs_mem.h>
#include <wx/treectrl.h>
#include <wx/font.h>
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
#ifndef __WXMSW__
    #include "Ressource/Image/MediaInfo.xpm"
#endif
//---------------------------------------------------------------------------

//***************************************************************************
// Constructor/Destructor
//***************************************************************************

//---------------------------------------------------------------------------
// Constructor
GUI_Main::GUI_Main(const wxPoint& pos, const wxSize& size, long style)
:wxFrame(NULL, -1, _T("MediaInfo"), pos, size, style)
{
    //Set the frame icon
    #ifdef WIN32 //Win32 use he 1st icon as file icon
    SetIcon(wxICON(aaaaaaaa));
    #else
    SetIcon(wxICON(MediaInfo));
    #endif

    //Menu and ToolBar
    Menu_Create();
    ToolBar_Create();

    //Status bar
    CreateStatusBar();

    //GUI
    Easy=NULL;
    Tree=NULL;
    HTML=NULL;
    Text=NULL;
    CenterOnScreen();

    //Core
    C=new Core;

    //Drag n drop
    #if wxUSE_DRAG_AND_DROP
        FD=new FileDrop(C, this);
        SetDropTarget(FD);
    #endif //wxUSE_DRAG_AND_DROP

    //Defaults
	Menu_View_Sheet->Enable(false);
	Menu_View_Tree->Enable (false);
	Menu_View_HTML->Check(); //Default to HTML with GUI.
    C->Menu_View_HTML(); //Default to HTML with Core.
	Menu_Debug_Demux_None->Check();
    Refresh_View();
}

//---------------------------------------------------------------------------
GUI_Main::~GUI_Main()
{
    SetDropTarget(NULL);
}

//***************************************************************************
// Events
//***************************************************************************

//---------------------------------------------------------------------------
void GUI_Main::OnSize(wxSizeEvent& WXUNUSED(event))
{
    if (IsShown())
        Resize_View();
}

//---------------------------------------------------------------------------
void GUI_Main::Resize_View()
{
    if (Easy)
    {
        Easy->SetSize(0, 0, GetClientSize().GetWidth(), GetClientSize().GetHeight());
        Easy_Select->SetSize(0, 0, Easy->GetClientSize().GetWidth(), 20);
        Easy_General->SetSize(2, Easy_Select->GetPosition().y+Easy_Select->GetSize().GetHeight()+1, Easy_Select->GetClientSize().GetWidth()-4, 100);
        Easy_General_Box->SetSize(0, 0, Easy_General->GetClientSize().GetWidth(), Easy_General->GetClientSize().GetHeight());
        Easy_General_Technical->SetSize(2, Easy_General_Box->GetCharHeight(), Easy_General->GetClientSize().GetWidth()/2-2, Easy_General->GetClientSize().GetHeight()-Easy_General_Box->GetCharHeight()-2);
        Easy_General_Tags->SetSize(Easy_General_Technical->GetPosition().x+Easy_General_Technical->GetSize().GetWidth()+1, Easy_General_Technical->GetPosition().y, Easy_General->GetClientSize().GetWidth()-Easy_General_Technical->GetPosition().x-Easy_General_Technical->GetSize().GetWidth()-4, Easy_General_Technical->GetClientSize().GetHeight());
        Easy_Video1->SetSize(Easy_General->GetPosition().x, Easy_General->GetPosition().y+Easy_General->GetSize().GetHeight(), Easy_General->GetClientSize().GetWidth()-Easy_General->GetPosition().x, 100);
        Easy_Audio1->SetSize(Easy_General->GetPosition().x, Easy_Video1->GetPosition().y+Easy_Video1->GetSize().GetHeight(), Easy_General->GetClientSize().GetWidth()/2-Easy_General->GetPosition().x, 100);
        Easy_Audio2->SetSize(Easy_Audio1->GetPosition().x+Easy_Audio1->GetSize().GetWidth()+1, Easy_Audio1->GetPosition().y, Easy_General->GetClientSize().GetWidth()-Easy_Audio1->GetPosition().x-Easy_Audio1->GetSize().GetWidth()-1, 100);
    }
    if (Tree)
        Tree->SetSize(0, 0, GetClientSize().GetWidth()-0, GetClientSize().GetHeight()-0);
    if (HTML)
        HTML->SetSize(1, 1, GetClientSize().GetWidth()-2, GetClientSize().GetHeight()-2);
    if (Text)
        Text->SetSize(0, 0, GetClientSize().GetWidth()-0, GetClientSize().GetHeight()-0);
}

//***************************************************************************
// Helpers
//***************************************************************************

//---------------------------------------------------------------------------
void GUI_Main::Refresh_View()
{
    C->Data_Prepare();

    switch (C->Kind_Get())
    {
        case Core::Kind_Easy  : Easy_Show ();  break;
        case Core::Kind_Sheet : Sheet_Show();  break;
        case Core::Kind_Tree  : Tree_Show ();  break;
        case Core::Kind_HTML  : HTML_Show ();  break;
        case Core::Kind_Text  : Text_Show ();  break;
    }
}

//---------------------------------------------------------------------------
void GUI_Main::Easy_Hide()
{
    if (!Easy)
        return;

    delete Easy; Easy=NULL;
}

//---------------------------------------------------------------------------
void GUI_Main::Easy_Show()
{
    Sheet_Hide();
    Tree_Hide();
    HTML_Hide();
    Text_Hide();

    if (!Easy)
    {
        Easy=new wxPanel(this);

        wxArrayString Choices;
        Choices.Add(_T("A"));
        Choices.Add(_T("B"));
        Easy_Select=new wxChoice(Easy, -1, wxDefaultPosition, wxDefaultSize, Choices);
        Easy_Select->SetSelection(0);

        Easy_General=new wxPanel(Easy);
        Easy_General_Box=new wxStaticBox(Easy_General, -1, _T("General"));
        Easy_General_Technical=new wxStaticText(Easy_General, -1, _T("XXX\r\nXXX"));
        Easy_General_Tags=new wxStaticText(Easy_General, -1, _T("YYY\r\nYYY"));
        Easy_Video1=new wxStaticBox(Easy, -1, _T("Video 1"));
        Easy_Audio1=new wxStaticBox(Easy, -1, _T("Audio 1"));
        Easy_Audio2=new wxStaticBox(Easy, -1, _T("Audio 2"));

        Resize_View();
    }

}

//---------------------------------------------------------------------------
void GUI_Main::Sheet_Hide()
{
}

//---------------------------------------------------------------------------
void GUI_Main::Sheet_Show()
{
    Easy_Hide();
    Tree_Hide();
    HTML_Hide();
    Text_Hide();
}

//---------------------------------------------------------------------------
void GUI_Main::Tree_Hide()
{
    if (!Tree)
        return;

    delete Tree; Tree=NULL;
}

//---------------------------------------------------------------------------
void GUI_Main::Tree_Show()
{
    Easy_Hide();
    Sheet_Hide();
    HTML_Hide();
    Text_Hide();

    if (!Tree)
    {
        Tree=new wxTreeCtrl(this, -1);
        Tree->SetWindowStyle(wxTR_HIDE_ROOT);
        Tree->SetSize(GetClientSize());
    }
    else
        Tree->DeleteAllItems();

	Tree->DeleteAllItems();
	wxTreeItemId RootID=Tree->AddRoot(wxT("Root"));

    String Text=C->Text_Get();
    if (Text.empty())
        return;
    wxTreeItemId ItemID[20];
    wxFont Font(8, wxFONTFAMILY_MODERN, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
    size_t Level=0;
    ItemID[0]=RootID;
    size_t Pos1=0, Pos2=0;
    while (Pos2!=(size_t)-1)
    {
        Pos2=Text.find('\n', Pos1);
        #ifdef WIN32
            #define SIZE 1
        #else
            #define SIZE 0
        #endif
        String Line=Text.substr(Pos1, Pos2-Pos1-SIZE);
        Level=Line.find_first_not_of(_T(' '), 9)-9;
        Line.erase(9, Level);
        ItemID[Level+1]=Tree->AppendItem(ItemID[Level], Line.c_str());
        Tree->SetItemFont(ItemID[Level+1], Font);
        Pos1=Pos2+1;
    }
}

//---------------------------------------------------------------------------
void GUI_Main::HTML_Hide()
{
    if (!HTML)
        return;

    delete HTML; HTML=NULL;
}

//---------------------------------------------------------------------------
void GUI_Main::HTML_Show()
{
    Easy_Hide();
    Sheet_Hide();
    Tree_Hide();
    Text_Hide();

    if (!HTML)
    {
        HTML=new wxHtmlWindow(this, -1, wxDefaultPosition, wxDefaultSize);
        Resize_View();
        int Size[7]={ 7, 8, 10, 12, 16, 22, 30};
        HTML->SetFonts(wxEmptyString, wxEmptyString, Size);
    }

    std::string Temp(wxConvUTF8.cWX2MB(C->Text_Get().c_str()));
    wxFileSystem::AddHandler(new wxMemoryFSHandler);
    wxMemoryFSHandler::AddFile(_T("index.html"), Temp.c_str(), Temp.size());
    HTML->LoadPage(_T("memory:index.html"));
    wxMemoryFSHandler::RemoveFile(_T("index.html"));
}

//---------------------------------------------------------------------------
void GUI_Main::Text_Hide()
{
    if (!Text)
        return;

    delete Text; Text=NULL;
}

//---------------------------------------------------------------------------
void GUI_Main::Text_Show()
{
    Easy_Hide();
    Sheet_Hide();
    Tree_Hide();
    HTML_Hide();

        //Text->SetDefaultStyle(Attr);

    if (!Text)
    {
        Text=new wxTextCtrl(this, -1, L"", wxDefaultPosition, wxDefaultSize, wxTE_READONLY|wxTE_MULTILINE|wxTE_RICH|wxTE_RICH2);
        Resize_View();
        wxFont Font;
        Font.SetFamily(wxFONTFAMILY_MODERN);
        wxTextAttr Attr;
        Attr.SetFont(Font);
        Text->SetDefaultStyle(Attr);
    }
    else
        Text->Clear();

    Text->WriteText(C->Text_Get().c_str());
}

