// File_Vorbis - Info for Vorbis files
// Copyright (C) 2007-2007 Jerome Martinez, Zen@MediaArea.net
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// Note : the buffer must be given in ONE call
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//---------------------------------------------------------------------------
// Compilation conditions
#include <MediaInfo/Setup.h>
#include <math.h>
#ifdef __BORLANDC__
    #pragma hdrstop
#endif
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
#if defined(MEDIAINFO_VORBIS_YES)
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
#include "MediaInfo/Audio/File_Vorbis.h"
//---------------------------------------------------------------------------

namespace MediaInfoLib
{

static const unsigned long mask[]=
{
    0x00000000,
    0x00000001, 0x00000003, 0x00000007, 0x0000000f,
    0x0000001f, 0x0000003f, 0x0000007f, 0x000000ff,
    0x000001ff, 0x000003ff, 0x000007ff, 0x00000fff,
    0x00001fff, 0x00003fff, 0x00007fff, 0x0000ffff,
    0x0001ffff, 0x0003ffff, 0x0007ffff, 0x000fffff,
    0x001fffff, 0x003fffff, 0x007fffff, 0x00ffffff,
    0x01ffffff, 0x03ffffff, 0x07ffffff, 0x0fffffff,
    0x1fffffff, 0x3fffffff, 0x7fffffff, 0xffffffff,
};

class BitStream_LE
{
    long endbyte;
    int  endbit;

    unsigned char *buffer;
    unsigned char *ptr;
    long storage;

public :
    BitStream_LE(unsigned char *buf,int bytes)
    {
      endbyte=0;
      endbit=0;
      buffer=buf;
      ptr=buf;
      storage=bytes;
    }

    int32u Get(int bits)
    {
        long ret;
        unsigned long m=mask[bits];

        bits+=endbit;

        if(endbyte+4>=storage){
        ret=-1L;
        if(endbyte*8+bits>storage*8)goto overflow;
        }

        ret=ptr[0]>>endbit;
        if(bits>8){
        ret|=ptr[1]<<(8-endbit);
        if(bits>16){
          ret|=ptr[2]<<(16-endbit);
          if(bits>24){
        ret|=ptr[3]<<(24-endbit);
        if(bits>32 && endbit){
          ret|=ptr[4]<<(32-endbit);
        }
          }
        }
        }
        ret&=m;

        overflow:

        ptr+=bits/8;
        endbyte+=bits/8;
        endbit=bits&7;
        return(ret);
    }

    void Skip(int bits)
    {
        Get(bits);
    }

    int32u Remain()
    {
        return 32;
    }

    int32u Offset_Get()
    {
        return ptr-buffer;
    }

    void Byte_Align()
    {
    }
};

int32u ilog(int32u Value)
{
    int32u ToReturn=0;
    while(Value)
    {
        ToReturn++;
        Value>>=1;
    }
    return(ToReturn);
}

//***************************************************************************
// Format
//***************************************************************************

//---------------------------------------------------------------------------
void File_Vorbis::Read_Buffer_Continue()
{
    Element_Size=Buffer_Size;
    ELEMENT(0, "Setup_Vorbis", Element_Size);

    //Parsing
    int32u codebook, codebook_dimensions, codebook_entries, ordered, codebook_lookup_type;
    int8u vorbis_codebook_count;
    BitStream_LE BS(Buffer+Buffer_Offset, Element_Size); //CC_BS_BEGIN();
    GET_BS (8, vorbis_codebook_count,                           vorbis_codebook_count);
    vorbis_codebook_count+=1;
    for (int Pos=0; Pos<vorbis_codebook_count; Pos++)
    {
        GET_BS (24, codebook,                                   codebook);
        if (codebook!=0x564342)
            return;
        GET_BS (16, codebook_dimensions,                        codebook_dimensions);
        GET_BS (24, codebook_entries,                           codebook_entries);
        GET_BS (1, ordered,                                     ordered);
        if (!ordered)
        {
            int32u sparse;
            GET_BS (1, sparse,                                  sparse);
            for (int32u Pos=0; Pos<codebook_entries; Pos++)
            {
                if (sparse)
                {
                    int32u flag;
                    GET_BS (1, flag,                             flag);
                    if (flag)
                    {
                        INFO_BS(5, length,                              length);
                    }
                }
                else
                {
                    INFO_BS(5, length,                              length);
                }
            }
        }
        else
        {
            SKIP_BS(5,                                          length);
            int32u num;
            for(int32u i=0; i<codebook_entries; )
            {
                GET_BS (ilog(codebook_entries-i), num,         num);
                for(int32u j=0; j<num && i<codebook_entries; j++, i++);
            }
        }
        GET_BS (4, codebook_lookup_type,                        codebook_lookup_type);
        if (codebook_lookup_type>2)
            return; //Not decodable
        if (codebook_lookup_type>0)
        {
            int32u codebook_value_bits;
            INFO_BS(32, codebook_minimum_value,                 codebook_minimum_value);
            INFO_BS(32, codebook_delta_value,                   codebook_delta_value);
            GET_BS ( 4, codebook_value_bits,                    codebook_value_bits);
            codebook_value_bits++;
            INFO_BS( 1, codebook_sequence_p,                    codebook_sequence_p);
            int32s vals;
            if (codebook_lookup_type==1)
            {
                vals=(int32u)floor(pow((float)codebook_entries,1.f/codebook_dimensions));
                while(1)
                {
                    int32u acc=1, acc1=1;
                    for(int32u i=0; i<codebook_dimensions; i++)
                    {
                        acc*=vals;
                        acc1*=vals+1;
                    }
                    if(acc<=codebook_entries && acc1>codebook_entries)
                        break;
                    else if(acc>codebook_entries)
                        vals--;
                    else
                        vals++;
                }
            }
            else //codebook_lookup_type==2
                vals=codebook_entries*codebook_dimensions;
            int32u codebook_multiplicands;
            for(int i=0; i<vals; i++)
                GET_BS (codebook_value_bits, codebook_multiplicands, codebook_multiplicands);
        }

    }

    //Time domain transforms
    int32u vorbis_time_count;
    GET_BS (6, vorbis_time_count,                               vorbis_time_count);
    for (int32u Pos=0; Pos<vorbis_time_count+1; Pos++)
        SKIP_BS(16,                                             zero);

    //Floors
    int32u vorbis_floor_count;
    GET_BS (6, vorbis_floor_count,                              vorbis_floor_count);
    for (int32u Pos=0; Pos<vorbis_floor_count; Pos++)
    {
        INFO_BS(16, vorbis_floor_types,                         vorbis_floor_types);
        if (Count_Get(Stream_Audio)==0)
        {
            Stream_Prepare(Stream_Audio);
            Fill("Codec_Settings/Floor", vorbis_floor_types);
            if (vorbis_floor_types==0)
                Fill("Codec_Settings", "Floor0");
        }
        //Must continue parsing...
    }
    BS_END_CANBEMORE();

    FLUSH();
    FINNISHED();
}

//***************************************************************************
// Information
//***************************************************************************

//---------------------------------------------------------------------------
void File_Vorbis::HowTo(stream_t StreamKind)
{
    switch (StreamKind)
    {
        case (Stream_General) :
            break;
        case (Stream_Video) :
            break;
        case (Stream_Audio) :
            break;
        case (Stream_Text) :
            break;
        case (Stream_Chapters) :
            break;
        case (Stream_Image) :
            break;
        case (Stream_Menu) :
            break;
        case (Stream_Max) :
            break;
    }
}

//***************************************************************************
// C++
//***************************************************************************

} //NameSpace

#endif //MEDIAINFO_MIDI_YES
