// File_Dvdv - Info for DVD objects (IFO) files
// Copyright (C) 2002-2007 Jerome Martinez, Zen@MediaArea.net
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// Information about DVD objects
// (.ifo files on DVD-Video)
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// Mainly from http://dvd.sourceforge.net/dvdinfo/ifo.html
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//---------------------------------------------------------------------------
// Compilation conditions
#include <MediaInfo/Setup.h>
#ifdef __BORLANDC__
    #pragma hdrstop
#endif
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
#if defined(MEDIAINFO_DVDV_YES)
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
#include "MediaInfo/Multiple/File_Dvdv.h"
#include <ZenLib/Utils.h>
#include <ZenLib/BitStream.h>
using namespace ZenLib;
//---------------------------------------------------------------------------

namespace MediaInfoLib
{

//---------------------------------------------------------------------------
const char*  IFO_VTS_Category[]=
{
    "Normal",
    "Karaoke",
};

const char*  IFO_CodecV[]=
{
    "MPV1",
    "MPV2",
    "",
    "",
};

const char*  IFO_Standard[]=
{
    "NTSC",
    "PAL",
    "",
    "",
};

const char*  IFO_AspectRatio[]=
{
    "1.333",
    "",
    "",
    "1.778",
};

const char*  IFO_BitRate_Mode[]=
{
    "VBR",
    "CBR",
};

const size_t IFO_Width[]=
{720, 704, 352, 352,   0,   0,   0,   0};

const size_t IFO_Height[4][8]=
{{480, 480, 480, 240,   0,   0,   0,   0}, //NTSC
 {576, 576, 576, 288,   0,   0,   0,   0}, //PAL
 {  0,   0,   0,   0,   0,   0,   0,   0}, //Unknown
 {  0,   0,   0,   0,   0,   0,   0,   0}, //Unknown
 };

const size_t IFO_FrameRate[]=
{25, 30};

const char*  IFO_CodecA[]=
{
    "AC3",
    "",
    "MPA1",
    "MPA2",
    "LPCM",
    "",
    "DTS",
    "SDDS",
};

const char*  IFO_ModeA[]=
{
    "",
    "Karaoke",
    "Surround",
    "",
};

const char*  IFO_ResolutionA[]=
{
    "16",
    "20",
    "24",
    "DRC",
};

const size_t IFO_Channels[]=
{1, 2, 3, 4, 6, 7, 0, 0};

const size_t IFO_SamplingRate[]=
{48000, 0, 0, 0, 0, 0, 0, 0};

const char*  IFO_Language_MoreA[]=
{
    "",
    "",
    "For visually impaired",
    "Director's comments",
    "Director's comments",
    "",
    "",
    "",
};

const char*  IFO_CodecT[]=
{
    "2-bit RLE",
    "",
    "",
    "",
    "",
    "",
    "",
    "",
};

const char*  IFO_Language_MoreT[]=
{
    "",
    "",
    "Large",
    "Children",
    "",
    "",
    "Large",
    "Children",
    "",
    "Forced",
    "",
    "",
    "",
    "Director comments",
    "Director comments large",
    "Director comments children",
};
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
#ifdef MEDIAINFO_AC3_YES
    extern const char*  AC3_ChannelPositions[];
#endif //MEDIAINFO_AC3_YES
//---------------------------------------------------------------------------

//***************************************************************************
// Format
//***************************************************************************

//---------------------------------------------------------------------------
void File_Dvdv::Read_Buffer_Init()
{
    //Buffer
    Element_Level=1;

    //Temp
    VTS_Attributes_AreHere=false;
    Program_Pos=0;
    Time_Pos=0;
}

//---------------------------------------------------------------------------
void File_Dvdv::Read_Buffer_Continue()
{
    if (File_Offset==0)
    {
        if (Buffer_Size<8)
            return;

        if (CC8(Buffer)!=CC8("DVDVIDEO"))
        {
            File_Offset=File_Size;
            return;
        }

        switch (CC4(Buffer+8))
        {
            case Dvdv::VMG : VMG(); break;
            case Dvdv::VTS : VTS(); break;
            default         :
                        File_Offset=File_Size;
                        return;
        }
    }

    //Others sectors...
    Sectors_Parse();
}

//***************************************************************************
// Elements
//***************************************************************************

#define SUBELEMENT(_BYTES, _ELEMENT) \
    { \
        FLUSH(); \
        Element_Level++; \
        size_t Buffer_Offset_Old=Buffer_Offset; \
        Buffer_Offset+=Stream_Pos; \
        _ELEMENT(); \
        Buffer_Offset=Buffer_Offset_Old; \
        Element_Level--; \
        ELEMENT_END(); \
        Stream_Pos+=_BYTES; \
    } \

//---------------------------------------------------------------------------
void File_Dvdv::VMG()
{
    ELEMENT(Element_Level-1, Dvdv::VTS, Element_Size);
    NAME("DVD Video - VMG");
    Buffer_Offset+=12;
    Element_Size=Buffer_Size-12;

    //Filling
    Stream_Prepare(Stream_General);
    General[0](_T("Format"))=_T("DVD Video");
    General[0](_T("Format/String"))=_T("DVD Video (Menu)");
    General[0](_T("Format/Extensions"))=_T("IFO");

    //Parsing
    int16u Version, Audio_Count, Text_Count;
    CC_BEGIN();
    SKIP_B4(                                                    Last sector of VMG set (last sector of BUP));
    SKIP_XX(12,                                                 Unknown);
    SKIP_B4(                                                    last sector of IFO);
    GET_B2 (Version,                                            version number); INFO(Ztring::ToZtring((Version&0x00F0)>>4)+_T(".")+Ztring::ToZtring(Version&0x000F));
    if (Version>0x001F)
        return;
    SKIP_B4(                                                    VMG category);
    SKIP_B2(                                                    number of volumes);
    SKIP_B2(                                                    volume number);
    SKIP_B1(                                                    side ID);
    SKIP_XX(19,                                                 Unknown);
    SKIP_B2(                                                    number of title sets);
    SKIP_LOCAL(32,                                              Provider ID);
    SKIP_B8(                                                    VMG POS);
    SKIP_XX(24,                                                 Unknown);
    SKIP_B4(                                                    end byte address of VMGI_MAT);
    SKIP_B4(                                                    start address of FP_PGC(First Play program chain));
    SKIP_XX(56,                                                 Unknown);
    SKIP_B4(                                                    start sector of Menu VOB);
    SKIP_B4(                                                    sector pointer to TT_SRPT (table of titles));
    SKIP_B4(                                                    sector pointer to VMGM_PGCI_UT (Menu Program Chain table));
    SKIP_B4(                                                    sector pointer to VMG_PTL_MAIT (Parental Management masks));
    SKIP_B4(                                                    sector pointer to VMG_VTS_ATRT (copies of VTS audio/sub-picture attributes));
    SKIP_B4(                                                    sector pointer to VMG_TXTDT_MG (text data));
    SKIP_B4(                                                    sector pointer to VMGM_C_ADT (menu cell address table));
    SKIP_B4(                                                    sector pointer to VMGM_VOBU_ADMAP (menu VOBU address map));
    SKIP_XX(32,                                                 Unknown);
    VTS_Attributes_AreHere=true; //now we have what we are looking for
    SUBELEMENT(2, Video);
    GET_B2 (Audio_Count,                                        number of audio streams in VMGM_VOBS);
    for (int16u Pos=0; Pos<8; Pos++)
    {
        if (Pos<Audio_Count)
            SUBELEMENT(8, Audio)
        else
            SKIP_XX(8,                                          Reserved for Audio);
    }
    SKIP_XX(16,                                                 Unknown);
    GET_B2 (Text_Count,                                         number of subpicture streams in VMGM_VOBS);
    for (int16u Pos=0; Pos<16; Pos++)
    {
        if (Pos<Text_Count)
            SUBELEMENT(6, Text)
        else
            SKIP_XX(6,                                          Reserved for Text);
    }
}

//---------------------------------------------------------------------------
void File_Dvdv::VTS()
{
    ELEMENT(Element_Level-1, Dvdv::VTS, Element_Size);
    NAME("DVD Video - VTS");
    Buffer_Offset+=12;
    Element_Size=Buffer_Size-12;
    Sectors.resize(File_Size/2048);

    //Filling
    Stream_Prepare(Stream_General);
    General[0](_T("Format"))=_T("DVD Video");
    General[0](_T("Format/String"))=_T("DVD Video");
    General[0](_T("Format/Extensions"))=_T("IFO");

    //Parsing
    int16u Version, Audio_Count, Text_Count;
    CC_BEGIN();
    SKIP_B4(                                                    Last sector of Title set (last sector of BUP));
    SKIP_XX(12,                                                 Unknown);
    SKIP_B4(                                                    last sector of IFO);
    GET_B2 (Version,                                            version number); INFO(Ztring::ToZtring((Version&0x00F0)>>4)+_T(".")+Ztring::ToZtring(Version&0x000F));
    if (Version>0x001F)
        return;
    INFO_B4(Category,                                           VTS category); if (Category<2) INFO(IFO_VTS_Category[Category]);
    SKIP_XX(90,                                                 Unknown);
    SKIP_B4(                                                    end byte address of VTS_MAT);
    SKIP_XX(60,                                                 Unknown);
    SKIP_B4(                                                    start sector of Menu VOB);
    SKIP_B4(                                                    start sector of Title Vob);
    GET_B4 (Sectors[Sector_VTS_PTT_SRPT],                       sector pointer to VTS_PTT_SRPT (Table of Titles and Chapters));
    GET_B4 (Sectors[Sector_VTS_PGCI],                           sector pointer to VTS_PGCI (Title Program Chain table));
    GET_B4 (Sectors[Sector_VTSM_PGCI_UT],                       sector pointer to VTSM_PGCI_UT (Menu Program Chain table));
    GET_B4 (Sectors[Sector_VTS_TMAPTI],                         sector pointer to VTS_TMAPTI (Time map));
    GET_B4 (Sectors[Sector_VTSM_C_ADT],                         sector pointer to VTSM_C_ADT (Menu cell address table));
    GET_B4 (Sectors[Sector_VTSM_VOBU_ADMAP],                    sector pointer to VTSM_VOBU_ADMAP(menu VOBU address map));
    GET_B4 (Sectors[Sector_VTS_C_ADT],                          sector pointer to VTS_C_ADT (Title set cell address table));
    GET_B4 (Sectors[Sector_VTS_VOBU_ADMAP],                     sector pointer to VTS_VOBU_ADMAP (Title set VOBU address map));
    SKIP_XX(24,                                                 Unknown);
    SUBELEMENT(2, Video);
    GET_B2 (Audio_Count,                                        number of audio streams in VMGM_VOBS);
    for (int16u Pos=0; Pos<8; Pos++)
    {
        if (Pos<Audio_Count)
            SUBELEMENT(8, Audio)
        else
            SKIP_XX(8,                                          Reserved for Audio);
    }
    SKIP_XX(16,                                                 Unknown);
    GET_B2 (Text_Count,                                         number of subpicture streams in VMGM_VOBS);
    for (int16u Pos=0; Pos<16; Pos++)
    {
        if (Pos<Text_Count)
            SUBELEMENT(6, Text)
        else
            SKIP_XX(6,                                          Reserved for Text);
    }
    SKIP_XX(74,                                                 Unknown);
    VTS_Attributes_AreHere=true; //now we have what we are looking for
    SUBELEMENT(2, Video);
    GET_B2 (Audio_Count,                                        number of audio streams in VMGM_VOBS);
    for (int16u Pos=0; Pos<8; Pos++)
    {
        if (Pos<Audio_Count)
            SUBELEMENT(8, Audio)
        else
            SKIP_XX(8,                                          Reserved for Audio);
    }
    SKIP_XX(16,                                                 Unknown);
    GET_B2 (Text_Count,                                         number of subpicture streams in VMGM_VOBS);
    for (int16u Pos=0; Pos<16; Pos++)
    {
        if (Pos<Text_Count)
            SUBELEMENT(6, Text)
        else
            SKIP_XX(6,                                          Reserved for Text);
    }
    SKIP_XX(2,                                                  Unknown);
    for (int16u Pos=0; Pos<8; Pos++)
    {
        if (Pos<Audio_Count)
            SKIP_XX(24,                                         Multichannel extension)
        else
            SKIP_XX(24,                                         Reserved for multichannel extension);
    }
}

//---------------------------------------------------------------------------
void File_Dvdv::Video()
{
    ELEMENT(Element_Level-1, "", 2);
    NAME("Video");

    //Parsing
    int32u Codec, Standard, AspectRatio, Resolution, BitRate_Mode;
    BS_BEGIN();
    GET_BS (2, Codec,                                           Coding mode); INFO(IFO_CodecV[Codec]);
    GET_BS (2, Standard,                                        Standard); INFO(IFO_Standard[Standard]);
    GET_BS (2, AspectRatio,                                     Aspect ratio); INFO(IFO_AspectRatio[AspectRatio]);
    INFO_BS(1, Pan,                                             Automatic Pan/Scan); INFO(Pan?"No":"Yes");
    INFO_BS(1, Letter,                                          Automatic Letterbox); INFO(Letter?"No":"Yes");
    SKIP_BS(1,                                                  CC for line 21 field 1 in GOP (NTSC only));
    SKIP_BS(1,                                                  CC for line 21 field 2 in GOP (NTSC only));
    GET_BS (3, Resolution,                                      Resolution); INFO(Ztring::ToZtring(IFO_Width[Resolution])+_T("x")+Ztring::ToZtring(IFO_Height[Standard][Resolution]));
    INFO_BS(1, Letterboxed,                                     Letterboxed); INFO(Letter?"Yes":"No");
    GET_BS (1, BitRate_Mode,                                    Bitrate mode); INFO(IFO_BitRate_Mode[BitRate_Mode]);
    INFO_BS(1, Camera,                                          Camera/Film); INFO(Letter?"Film":"Camera");

    //Filling
    if (VTS_Attributes_AreHere)
    {
        Stream_Prepare(Stream_Video);
        Fill("Codec", IFO_CodecV[Codec]);
        Fill("AspectRatio", IFO_AspectRatio[AspectRatio]);
        Fill("Width", IFO_Width[Resolution]);
        Fill("Height", IFO_Height[Standard][Resolution]);
        Fill("FrameRate", IFO_FrameRate[Standard]);
        Fill("BitRate_Mode", IFO_BitRate_Mode[BitRate_Mode]);
    }
}

//---------------------------------------------------------------------------
void File_Dvdv::Audio()
{
    ELEMENT(Element_Level-1, "", 8);
    NAME("Audio");

    //Parsing
    int32u Codec, LanguageType, Mode, Resolution, SamplingRate, Channels, Language1, Language2, Language_Extension;
    BS_BEGIN();
    GET_BS (3, Codec,                                           Coding mode); INFO(IFO_CodecA[Codec]);
    INFO_BS(1, MultiChannel,                                    Multichannel extension present); INFO(MultiChannel?"Yes":"No");
    GET_BS (2, LanguageType,                                    Language type); INFO(LanguageType==1?"2CC":"Unknown");
    GET_BS (2, Mode,                                            Application mode); INFO(IFO_ModeA[Mode]);
    GET_BS (2, Resolution,                                      Resolution); if (Codec==2 || Codec==3) INFO(IFO_ResolutionA[Resolution]); else if (Codec==4) INFO(Mode?"DRC":"No DRC");
    GET_BS (2, SamplingRate,                                    Sampling rate); INFO(Ztring::ToZtring(IFO_SamplingRate[SamplingRate]));
    GET_BS (4, Channels,                                        Channels);
    GET_BS (8, Language1,                                       Language code 1);
    GET_BS (8, Language2,                                       Language code 2);
    SKIP_BS(8,                                                  Language code reserved);
    GET_BS (8, Language_Extension,                              Language extension); if (Language_Extension<8) INFO(IFO_Language_MoreA[Language_Extension]);
    switch (Mode)
    {
        case 1 : //Karaoke
            {
            SKIP_BS(1,                                          Zero);
            INFO_BS(3, ChannelsK,                               Channels);
                #ifdef MEDIAINFO_AC3_YES
                    INFO(AC3_ChannelPositions[ChannelsK]);
                #endif //MEDIAINFO_AC3_YES
            SKIP_BS(2,                                          Version);
            INFO_BS(1, MC,                                      MC intro present); INFO(MC?"Yes":"No");
            INFO_BS(1, Duet,                                    Duet); INFO(Duet?"Duet":"Solo");
            }
            break;
        case 2 : //Surround
            {
            SKIP_BS(4,                                          Reserved);
            INFO_BS(1, DolbyDecode,                             Suitable for Dolby surround decoding); INFO(DolbyDecode?"Yes":"No");
            SKIP_BS(3,                                          Reserved);
            }
            break;
    default:
            {
            SKIP_BS(8,                                          Reserved);
            }
    }

    //Filling
    if (VTS_Attributes_AreHere)
    {
        Stream_Prepare(Stream_Audio);
        Fill("Codec", IFO_CodecA[Codec]);
        Fill("SamplingRate", IFO_SamplingRate[SamplingRate]);
        Fill("Channel(s)", IFO_Channels[Channels]);
        if (Codec==2 || Codec==3)
            Fill("Resolution", IFO_ResolutionA[Resolution]);
        else if (Codec==4 && Mode)
            Fill("Resolution", "DRC");
        Ztring Lang; Lang+=(char)(Language1); Lang+=(char)(Language2);
        Fill("Language", Lang);
        if (Language_Extension<8)
            Fill("Language_More", IFO_Language_MoreA[Language_Extension]);
    }
}

//---------------------------------------------------------------------------
void File_Dvdv::Text()
{
    ELEMENT(Element_Level-1, "", 6);
    NAME("Text");

    //Parsing
    int32u Codec, LanguageType, Language1, Language2, Language_Extension;
    BS_BEGIN();
    GET_BS (3, Codec,                                           Coding mode); INFO(IFO_CodecT[Codec]);
    SKIP_BS(3,                                                  Reserved);
    GET_BS (2, LanguageType,                                    Language type); INFO(LanguageType==1?"2CC":"Unknown");
    SKIP_BS(8,                                                  Reserved);
    GET_BS (8, Language1,                                       Language code 1);
    GET_BS (8, Language2,                                       Language code 2);
    SKIP_BS(8,                                                  Language code reserved);
    GET_BS (8, Language_Extension,                              Language extension); if (Language_Extension<16) INFO(IFO_Language_MoreT[Language_Extension]);

    //Filling
    if (VTS_Attributes_AreHere)
    {
        Stream_Prepare(Stream_Text);
        Fill("Codec", IFO_CodecT[Codec]);
        Ztring Lang; Lang+=(char)(Language1); Lang+=(char)(Language2);
        Fill("Language", Lang);
        if (Language_Extension<16)
            Fill("Language_More", IFO_Language_MoreT[Language_Extension]);
    }
}

//***************************************************************************
// Sectors
//***************************************************************************

//---------------------------------------------------------------------------
void File_Dvdv::Sectors_Parse ()
{
    Buffer_Offset=0; //Sometimes, it is not set to a multiple of 2048
    while(Buffer_Offset+2048<=Buffer_Size)
    {
        Element_Level++;

        int32u Sector_Pos=(File_Offset+Buffer_Offset)/2048;
        if (Sector_Pos>0 && Sectors.size()>0)
        {
            Element_Size=Buffer_Size-Buffer_Offset;
            ELEMENT(Element_Level-1, "", 2048);
                 if (0);
            else if (Sector_Pos==Sectors[Sector_VTS_PTT_SRPT]) VTS_PTT_SRPT();
            else if (Sector_Pos==Sectors[Sector_VTS_PGCI]) VTS_PGCI();
            else if (Sector_Pos==Sectors[Sector_VTSM_PGCI_UT]) VTSM_PGCI_UT();
            else if (Sector_Pos==Sectors[Sector_VTS_TMAPTI]) VTS_TMAPTI();
            else if (Sector_Pos==Sectors[Sector_VTSM_C_ADT]) VTSM_C_ADT();
            else if (Sector_Pos==Sectors[Sector_VTSM_VOBU_ADMAP]) VTSM_VOBU_ADMAP();
            else if (Sector_Pos==Sectors[Sector_VTS_C_ADT]) VTS_C_ADT();
            else if (Sector_Pos==Sectors[Sector_VTS_VOBU_ADMAP]) VTS_VOBU_ADMAP();
            else {NAME("Unknown sector"); Element_Size=0;};
        }
        else
            Element_Size=0;

        FLUSH();
        Buffer_Offset+=2048*(Element_Size/2048+1);
        Element_Level--;
    }
}

//---------------------------------------------------------------------------
void File_Dvdv::VTS_PTT_SRPT ()
{
    NAME("Titles and Chapters");

    //Parsing
    int32u Offset;
    CC_BEGIN();
    SKIP_B2(                                                    Number of titles);
    SKIP_B2(                                                    Reserved);
    GET_B4 (Element_Size,                                       End address (last byte of last VTS_PTT));
    Element_Size++; //Last byte
    GET_B4 (Offset,                                             Offset to PTT for title 1);
    if (Offset-12>0)
        SKIP_XX(Offset-12,                                      Unknown);
    DETAILLEVEL_SET(1.0);
    while (Stream_Pos<Element_Size)
    {
        //VTS_PTT
        SKIP_B2(                                                Program Chain (PGCN));
        SKIP_B2(                                                Program (PGN));
    }
}

//---------------------------------------------------------------------------
void File_Dvdv::VTS_PGCI ()
{
    NAME("Title Program Chain table");

    //Parsing
    CC_BEGIN();
    SKIP_B2(                                                    Number of Program Chains);
    SKIP_B2(                                                    Reserved);
    GET_B4 (Element_Size,                                       End address (last byte of last PGC) relative to VTS_PGCI);
    Element_Size++; //Last byte
}

//---------------------------------------------------------------------------
void File_Dvdv::VTSM_PGCI_UT ()
{
    NAME("Menu Program Chain table");

    //Parsing
    int32u Offset;
    CC_BEGIN();
    SKIP_B2(                                                    Number of Language Units);
    SKIP_B2(                                                    Reserved);
    GET_B4 (Element_Size,                                       End address (last byte of last PGC in last LU) relative to VTSM_PGCI_UT);
    Element_Size++; //Last byte
    SKIP_C2(                                                    Language);
    SKIP_B1(                                                    Reserved);
    SKIP_B1(                                                    Menu existence flags);
    GET_B4 (Offset,                                             Offset to VTSM_LU relative to VTSM_PGCI_UT);
    if (Offset-12>0)
        SKIP_XX(Offset-12,                                      Unknown);
}

//---------------------------------------------------------------------------
void File_Dvdv::VTS_TMAPTI ()
{
    NAME("Time map");

    //Parsing
    int32u Offset;
    CC_BEGIN();
    SKIP_B2(                                                    Number of program chains);
    SKIP_B2(                                                    Reserved);
    GET_B4 (Element_Size,                                       End address (last byte of last VTS_TMAP));
    Element_Size++; //Last byte
    GET_B4 (Offset,                                             Offset to VTS_TMAP 1);
    if (Offset-12>0)
        SKIP_XX(Offset-12,                                      Unknown);
    //DETAILLEVEL_SET(1.0);
    size_t Program_Pos=0;
    while (Stream_Pos<Element_Size)
    {
        //VTS_TMAP
        Sectors_Times.resize(Program_Pos+1);
        Sectors_Times_SecondsPerTime.resize(Program_Pos+1);
        int32u Count;
        GET_B1 (Sectors_Times_SecondsPerTime[Program_Pos],      Time unit (seconds));
        SKIP_B1(                                                Unknown);
        GET_B2 (Count,                                          Number of entries in map);
        Sectors_Times[Program_Pos].resize(Count);
        for (int32u Pos=0; Pos<Count; Pos++)
        {
            GET_B4 (Sectors_Times[Program_Pos][Pos],            Sector offset within VOBS of nearest VOBU); INFO(Ztring().Duration_From_Milliseconds((Pos+1)*Sectors_Times_SecondsPerTime[Program_Pos]));
            Sectors_Times[Program_Pos][Pos]=Sectors_Times[Program_Pos][Pos]&0x7FFFFFFF; //bit 31 is set if VOBU time codes are discontinuous with previous
        }
    }
}

//---------------------------------------------------------------------------
void File_Dvdv::VTSM_C_ADT ()
{
    NAME("Menu cell address table");

    //Parsing
    CC_BEGIN();
    SKIP_B2(                                                    Number of cells);
    SKIP_B2(                                                    Reserved);
    GET_B4 (Element_Size,                                       End address (last byte of last entry));
    Element_Size++; //Last byte
    DETAILLEVEL_SET(1.0);
    while (Stream_Pos<Element_Size)
    {
        //ADT
        SKIP_B2(                                                VOBidn);
        SKIP_B1(                                                CELLidn);
        SKIP_B1(                                                Unknown);
        SKIP_B4(                                                Starting sector within VOB);
        SKIP_B4(                                                Ending sector within VOB);
    }
}

//---------------------------------------------------------------------------
void File_Dvdv::VTSM_VOBU_ADMAP ()
{
    NAME("Menu VOBU address map");

    //Parsing
    CC_BEGIN();
    GET_B4 (Element_Size,                                       End address (last byte of last entry));
    Element_Size++; //Last byte
    DETAILLEVEL_SET(1.0);
    while (Stream_Pos<Element_Size)
    {
        //ADMAP
        SKIP_B4(                                                Starting sector within VOB of first VOBU);
    }
}

//---------------------------------------------------------------------------
void File_Dvdv::VTS_C_ADT ()
{
    NAME("Title set cell address table");

    //Parsing
    CC_BEGIN();
    SKIP_B2(                                                    Number of cells);
    SKIP_B2(                                                    Reserved);
    GET_B4 (Element_Size,                                       End address (last byte of last entry));
    Element_Size++; //Last byte
    while (Stream_Pos<Element_Size)
    {
        //ADT
        int32u VOBidn, CELLidn, Start, End;
        GET_B2 (VOBidn,                                         VOBidn);
        //Filling
        if (Count_Get(Stream_Chapters)<VOBidn)
            Stream_Prepare(Stream_Chapters);
        GET_B1 (CELLidn,                                        CELLidn);
        SKIP_B1(                                                Unknown);
        GET_B4 (Start,                                          Starting sector within VOB); INFO(Time_ADT(Start));
        //Filling
        Fill(Ztring::ToZtring(CELLidn).To_Local().c_str(), Time_ADT(Start));
        GET_B4 (End,                                            Ending sector within VOB); INFO(Time_ADT(End));
    }
}

//---------------------------------------------------------------------------
void File_Dvdv::VTS_VOBU_ADMAP ()
{
    NAME("Title set VOBU address map");

    //Parsing
    CC_BEGIN();
    GET_B4 (Element_Size,                                       End address (last byte of last entry));
    Element_Size++; //Last byte
    DETAILLEVEL_SET(1.0);
    while (Stream_Pos<Element_Size)
    {
        //ADMAP
        SKIP_B4(                                                Starting sector within VOB of first VOBU);
    }
}

//***************************************************************************
// Helpers
//***************************************************************************

//---------------------------------------------------------------------------
Ztring File_Dvdv::Time_ADT(int32u Value)
{
    while (Time_Pos<Sectors_Times[Program_Pos].size() && Sectors_Times[Program_Pos][Time_Pos]<Value)
        Time_Pos++;
    if (Time_Pos<Sectors_Times[Program_Pos].size())
    {
        int32u Time=(Time_Pos+1)*Sectors_Times_SecondsPerTime[Program_Pos]*1000;
        float32 Part;
        //True time is between Time and Time+Sectors_Times_SecondsPerTime, finding where...
        int32u Sectors_Count;
        if (Time_Pos==0)
            Sectors_Count=Sectors_Times[Program_Pos][Time_Pos];
        else
            Sectors_Count=Sectors_Times[Program_Pos][Time_Pos]-Sectors_Times[Program_Pos][Time_Pos-1];
        Part=(Sectors_Times[Program_Pos][Time_Pos]-Value); //Count of more sectors after
        Part/=Sectors_Count; //Percentage
        Time=(int32u)((Time_Pos+1-Part)*Sectors_Times_SecondsPerTime[Program_Pos]*1000);

        return Ztring().Duration_From_Milliseconds(Time*1000);
    }
    else
    {
        int32u Time=(Time_Pos+1)*Sectors_Times_SecondsPerTime[Program_Pos]*1000;
        float32 Part;
        //True time is between Time and Time+Sectors_Times_SecondsPerTime, finding where... but with the last offset diffrence
        int32u Sectors_Count=Sectors_Times[Program_Pos][Time_Pos-1]-Sectors_Times[Program_Pos][Time_Pos-2];
        Part=((int32s)Sectors_Times[Program_Pos][Time_Pos-1])-((int32s)Value); //Count of more sectors after
        Part/=Sectors_Count; //Percentage
        Part+=1; //We were one offset less
        Time=(int32u)((Time_Pos+1-Part)*Sectors_Times_SecondsPerTime[Program_Pos]*1000);

        return Ztring().Duration_From_Milliseconds(Time*1000);
    }
}


//***************************************************************************
// Information
//***************************************************************************

//---------------------------------------------------------------------------
void File_Dvdv::HowTo(stream_t StreamKind)
{
    switch (StreamKind)
    {
        case (Stream_General) :
            Fill_HowTo("Format", "R");
            break;
        case (Stream_Video) :
            Fill_HowTo("Codec", "R");
            Fill_HowTo("BitRate", "R");
            Fill_HowTo("Width", "R");
            Fill_HowTo("Height", "R");
            Fill_HowTo("AspectRatio", "R");
            Fill_HowTo("FrameRate", "R");
            break;
        case (Stream_Audio) :
            Fill_HowTo("Codec", "R");
            Fill_HowTo("BitRate", "R");
            Fill_HowTo("Channel(s)", "R");
            Fill_HowTo("SamplingRate", "R");
            Fill_HowTo("Resolution", "R");
            Fill_HowTo("Language", "R");
            Fill_HowTo("Language_More", "R");
            Fill_HowTo("Codec", "R");
            Fill_HowTo("Language", "R");
            Fill_HowTo("Language_More", "R");
        case (Stream_Text) :
            break;
        case (Stream_Chapters) :
            break;
        case (Stream_Image) :
            break;
        case (Stream_Menu) :
            break;
        case (Stream_Max) :
            break;
    }
}

} //NameSpace

#endif //MEDIAINFO_DVDV_YES

