// File_Ogg - Info for ogg files
// Copyright (C) 2002-2007 Jerome Martinez, Zen@MediaArea.net
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//---------------------------------------------------------------------------
// Compilation conditions
#include <MediaInfo/Setup.h>
#ifdef __BORLANDC__
    #pragma hdrstop
#endif
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
#if defined(MEDIAINFO_OGG_YES)
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
#include "MediaInfo/Multiple/File_Ogg.h"
#include "MediaInfo/Multiple/File_Ogg_SubElement.h"
#include <ZenLib/BitStream.h>
using namespace ZenLib;
//---------------------------------------------------------------------------

namespace MediaInfoLib
{

//***************************************************************************
// Format
//***************************************************************************

//---------------------------------------------------------------------------
void File_Ogg::Read_Buffer_Init()
{
    //Temp - Global
    StreamsToDo=0;
}

//---------------------------------------------------------------------------
void File_Ogg::Read_Buffer_Unsynched()
{
    if (Config.Details_Get())
    {
        Details_Add_Error("Will resync...");
    }

    std::map<int32u, int64u>::iterator absolute_granule_position_Temp=absolute_granule_position.begin();
    while (absolute_granule_position_Temp!=absolute_granule_position.end())
    {
        absolute_granule_position_Temp->second=0;
        absolute_granule_position_Temp++;
    }
}

//---------------------------------------------------------------------------
void File_Ogg::Read_Buffer_Continue()
{
    if (!Synched && !Synchronize())
    {
        File_Offset=File_Size;
        return;
    }

    //Normal parsing
    while (Buffer_Parse());
}

//---------------------------------------------------------------------------
void File_Ogg::Read_Buffer_Finalize()
{
    std::map<int32u, int64u>::iterator absolute_granule_position_Temp=absolute_granule_position.begin();
    while (absolute_granule_position_Temp!=absolute_granule_position.end())
    {
        if (absolute_granule_position_Resolution[absolute_granule_position_Temp->first])
            Fill(StreamKind[absolute_granule_position_Temp->first], StreamPos[absolute_granule_position_Temp->first], "PlayTime", float64_int64s(((float64)(absolute_granule_position_Temp->second))*1000/absolute_granule_position_Resolution[absolute_granule_position_Temp->first]));
        absolute_granule_position_Temp++;
    }

    Fill(Stream_General, 0, "Format", "OGG");
}

//***************************************************************************
// Buffer
//***************************************************************************

//---------------------------------------------------------------------------
bool File_Ogg::Buffer_Parse()
{
    //Enough buffer?
    if (Buffer_Offset+40>Buffer_Size)
        return false;

    //Configure defines : must return false
    #undef INTEGRITY_RETURNVALUE
    #define INTEGRITY_RETURNVALUE false

    //Parsing
    Element_Size=Buffer_Size;
    int64u absolute_granule_position_Temp;
    int32u capture_pattern, page_sequence_no, packet_lacing_value;
    int16u total_page_size;
    int8u stream_structure_version, flags, page_segments;
    CC_BEGIN();
    GET_C4 (capture_pattern,                                    capture_pattern);
    if (capture_pattern!=Ogg::OggS)
    {
        File_Offset=File_Size;
        return false;
    }
    GET_L1 (stream_structure_version,                           stream_structure_version);
    GET_L1 (flags,                                              header_type_flag);
        GET_FLAGS (flags, 0, continued,                         continued packet);
        SKIP_FLAGS(flags, 1,                                    first page of logical bitstream (bos));
        SKIP_FLAGS(flags, 2,                                    last page of logical bitstream (eos));
    GET_L8 (absolute_granule_position_Temp,                     absolute granule position);
    GET_L4 (Element_Name,                                       stream serial number);
    absolute_granule_position[Element_Name]=absolute_granule_position_Temp;
    GET_L4 (page_sequence_no,                                   page sequence no);
    SKIP_L4(                                                    page checksum);
    GET_L1 (page_segments,                                      page_segments);
    Element_Size=27+page_segments+1;
    total_page_size=0;
    for (int8u Pos=0; Pos<page_segments; Pos++)
    {
        GET_L1 (packet_lacing_value,                            packet lacing value);
        total_page_size+=packet_lacing_value;
    }
    if (!continued)
    {
        PEEK_L1(packet_type); //Only for information, will be really used later
    }
    CC_END_CANBEMORE();

    //UnConfigure defines : must return nothing
    #undef INTEGRITY_RETURNVALUE
    #define INTEGRITY_RETURNVALUE

    //Filling
    Element_HeaderSize=27+page_segments;
    Element_Size=total_page_size;
    Element_Next=File_Offset+Buffer_Offset+Element_HeaderSize+Element_Size;

    //Parsing
    if (!Element_Parse())
        return false; //Not enough bytes

    //Next element
    Buffer_Offset=(size_t)(Element_Next-File_Offset);

    return true;
}

//---------------------------------------------------------------------------
bool File_Ogg::Element_Parse()
{
    //Enough size?
    if (Element_Next-File_Offset>Buffer_Size)
        return false;

    ELEMENT(0, Ztring::ToZtring(Element_Name, 16), Element_Size);
    NAME (Ztring::ToZtring(packet_type, 16));
    if (continued)
        NAME("Continue");

    Buffer_Offset+=Element_HeaderSize;

    if (Ogg_Stream[Element_Name]==NULL)
    {
        Ogg_Stream[Element_Name]=new File_Ogg_SubElement;
        StreamsToDo++;
    }

    if (Ogg_Stream[Element_Name]->File_Offset!=File_Size)
    {
        Open_Buffer_Init(Ogg_Stream[Element_Name], File_Size, File_Offset+Buffer_Offset);
        if (!continued)
            Open_Buffer_Continue(Ogg_Stream[Element_Name], Buffer+Buffer_Offset, 0); //Purge old datas
        if (continued || Ogg_Stream[Element_Name]->File_Offset!=Ogg_Stream[Element_Name]->File_Size)
            Open_Buffer_Continue(Ogg_Stream[Element_Name], Buffer+Buffer_Offset, Element_Size);

        if (Ogg_Stream[Element_Name]->File_Offset==File_Size)
        {
            Merge(*Ogg_Stream[Element_Name]);
            Merge(*Ogg_Stream[Element_Name], Stream_General, 0, 0);
            StreamKind[Element_Name]=((File_Ogg_SubElement*)Ogg_Stream[Element_Name])->StreamKind;
            StreamPos[Element_Name]=Count_Get(StreamKind[Element_Name])-1;
            absolute_granule_position_Resolution[Element_Name]=((File_Ogg_SubElement*)Ogg_Stream[Element_Name])->absolute_granule_position_Resolution;
            StreamsToDo--;
        }
    }

    if (StreamsToDo==0)
    {
        if (File_Size>2*256*1024 && File_Offset+Buffer_Offset<File_Size-256*1024)
        {
            Details_Add_Error("---------------------------------------");
            Details_Add_Error("---   OGG, Jumping to end of file   ---");
            Details_Add_Error("---------------------------------------");
            File_GoTo=File_Size-256*1024;
            return false;
        }
    }

    FLUSH();
    return true;
}


/*
void File_Ogg::Read_Buffer_Continue()
{
    ShouldStop=false;

    //Integrity test
    if (Buffer_Size<4 || CC4(Buffer)!=CC4("OggS"))
    {
        File_Offset=File_Size;
        return;
    }

    Stream_Prepare(Stream_General);
    Fill("Format", "Ogg");

    //Buffer
    while (!ShouldStop)
    {
        ChunkHeader_Analyse();
        if (!ShouldStop)
        {
            ChunkData_Analyse();
        }
    }

    //Some headers my have not comments, but Analysing start withs comments... We analyse this chunks now
    while (!ChunkHeader_ID.empty())
        Identification_Analyse(ChunkHeader_ID[0]);


    //Finding last frame time
    Buffer_Offset=End_Size-14;
    while (Buffer_Offset>0 && CC4(End+Buffer_Offset)!=CC4("OggS"))
        Buffer_Offset--;
    if (Buffer_Offset==0)
        return 1; //Time not found, but begin is OK...
    int64u Size=LittleEndian2int64u(End+Buffer_Offset+6);
    Ztring ID; ID.From_Number(LittleEndian2int32u(End+Buffer_Offset+14));
    size_t PlayTime=0;
    for (size_t Pos=0; Pos<Audio.size(); Pos++)
        if (Get(Stream_Audio, Pos, _T("ID"))==ID && Get(Stream_Audio, Pos, _T("SamplingRate")).To_int32u()!=0)
            PlayTime=Size*1000/Get(Stream_Audio, Pos, _T("SamplingRate")).To_int32u();
    for (size_t Pos=0; Pos<Video.size(); Pos++)
        if (Get(Stream_Video, Pos, _T("ID"))==ID && Get(Stream_Video, Pos, _T("FrameRate")). To_float32()!=0)
            PlayTime=Size*1000/Get(Stream_Video, Pos, _T("FrameRate")).To_int32u();
    if (PlayTime)
        Fill(Stream_General, 0, "PlayTime", PlayTime);


    //No need of more
    File_Offset=File_Size;
}
*/


//***************************************************************************
// Helpers
//***************************************************************************

//---------------------------------------------------------------------------
bool File_Ogg::Synchronize()
{
    while (Buffer_Offset+4<Buffer_Size)
    {
        if (CC4(Buffer+Buffer_Offset)==Ogg::OggS)
        {
            Synched=true;
            return true;
        }
        Buffer_Offset++;
    }

    //Sync not found
    return false;
}

//***************************************************************************
// Information
//***************************************************************************

//---------------------------------------------------------------------------
void File_Ogg::HowTo(stream_t StreamKind)
{
    switch (StreamKind)
    {
        case (Stream_General) :
            Fill_HowTo("Format", "R");
            Fill_HowTo("BitRate", "R");
            Fill_HowTo("PlayTime", "R");
            Fill_HowTo("Album", "R ALBUM");
            Fill_HowTo("Movie", "R TITLE");
            Fill_HowTo("Movie/More", "R VERSION");
            Fill_HowTo("Track", "R TITLE");
            Fill_HowTo("Track/More", "R VERSION");
            Fill_HowTo("Artist", "R ARTIST");
            Fill_HowTo("Encoded_Date", "R DATE");
            Fill_HowTo("Recorded_Location", "R LOCATION");
            Fill_HowTo("Copyright", "R COPYRIGHT");
            Fill_HowTo("Comment", "R DESCRIPTION or COMMENT");
            break;
        case (Stream_Video) :
            Fill_HowTo("Codec", "R");
            Fill_HowTo("BitRate", "R");
            Fill_HowTo("Width", "R");
            Fill_HowTo("Height", "R");
            Fill_HowTo("AspectRatio", "R");
            Fill_HowTo("FrameRate", "R");
            break;
        case (Stream_Audio) :
            Fill_HowTo("Codec", "R");
            Fill_HowTo("BitRate", "R");
            Fill_HowTo("Channel(s)", "R");
            Fill_HowTo("SamplingRate", "R");
            Fill_HowTo("Language", "R LANGUAGE");
            Fill_HowTo("Language/String", "R");
            Fill_HowTo("Language/Info", "R");
            break;
        case (Stream_Text) :
            Fill_HowTo("Codec", "R");
            Fill_HowTo("Language", "R");
            Fill_HowTo("Language/String", "R LANGUAGE");
            Fill_HowTo("Language/Info", "R");
            break;
        case (Stream_Chapters) :
            Fill_HowTo("Total", "R");
            break;
        case (Stream_Image) :
            break;
        case (Stream_Menu) :
            break;
        case (Stream_Max) :
            break;
    }
}

} //NameSpace

#endif //MEDIAINFO_OGG_YES
