// File_Ogg_SubElement - Info for OGG files
// Copyright (C) 2007-2007 Jerome Martinez, Zen@MediaArea.net
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// Information about MPEG Transport Stream files, Program Map Section
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//---------------------------------------------------------------------------
#ifndef MediaInfo_Ogg_Setup_VorbisH
#define MediaInfo_Ogg_Setup_VorbisH
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
#include "MediaInfo/File__Base.h"
//---------------------------------------------------------------------------

namespace MediaInfoLib
{

//***************************************************************************
// Class File_Ogg_SubElement
//***************************************************************************

class File_Ogg_SubElement : public File__Base
{
public :
    stream_t StreamKind;
    int64u absolute_granule_position_Resolution;

protected :
    //Formats
    void Read_Buffer_Init ();
    void Read_Buffer_Continue ();

private :
    void Buffer_Parse();
    bool Element_Parse();
    int8u  Element_Name;
    size_t Element_Size;
    size_t OldSize;

    //Elements
    void Identification();
    void Identification_vorbis();
    void Identification_theora();
    void Identification_video();
    void Identification_audio();
    void Identification_text();
    void Comment();
    void Comment_vorbis();
    void Comment_theora();
    void Comment_video();
    void Comment_audio();
    void Comment_text();
    void Setup();
    void Setup_vorbis();

    //Temp
    File__Base* Setup_Vorbis;
};

#ifdef __BORLANDC__ //Borland converts int64u to int32u without error or warning
    #define OGG_ID(NAME, PART1, PART2) \
        const int32u NAME##1=0x##PART1; \
        const int32u NAME##2=0x##PART2; \

#else //__BORLANDC__
    #define OGG_ID(NAME, PART1, PART2) \
        const int64u NAME=0x##PART1##PART2##LL; \

#endif //__BORLANDC__

namespace Ogg
{
    const int32u OggS=0x4F676753;

    OGG_ID(vorbis, 766F, 72626973);
    OGG_ID(theora, 7468, 656F7261);
    OGG_ID(video,  7669, 64656F00);
    OGG_ID(audio,  6175, 64696F00);
    OGG_ID(text,   7465, 78740000);
};

} //NameSpace

#endif
